class CorpusDisplay extends RessourceDisplay {
  constructor(displayElement) {
    super(displayElement);
    this.corpusId = displayElement.dataset.corpusId;
    this.user.eventListeners.corpus.addEventListener((eventType, payload) => this.eventHandler(eventType, payload), this.corpusId);
  }

  init() {
    for (let exportCorpusTriggerElement of this.displayElement.querySelectorAll('.export-corpus-trigger')) {exportCorpusTriggerElement.addEventListener('click', () => this.requestCorpusExport());}
    nopaque.appClient.socket.on(`export_corpus_${this.user.data.corpora[this.corpusId].id}`, () => this.downloadCorpus());
    this.setCreationDate(this.user.data.corpora[this.corpusId].creation_date);
    this.setDescription(this.user.data.corpora[this.corpusId].description);
    this.setLastEditedDate(this.user.data.corpora[this.corpusId].last_edited_date);
    this.setStatus(this.user.data.corpora[this.corpusId].status);
    this.setTitle(this.user.data.corpora[this.corpusId].title);
    this.setTokenRatio(this.user.data.corpora[this.corpusId].current_nr_of_tokens, this.user.data.corpora[this.corpusId].max_nr_of_tokens);
  }

  patch(patch) {
    let re;
    for (let operation of patch) {
      switch(operation.op) {
        case 'replace':
          // Matches: /jobs/{this.job.id}/status
          re = new RegExp('^/corpora/' + this.user.data.corpora[this.corpusId].id + '/last_edited_date');
          if (re.test(operation.path)) {this.setLastEditedDate(operation.value); break;}
          // Matches: /jobs/{this.job.id}/status
          re = new RegExp('^/corpora/' + this.user.data.corpora[this.corpusId].id + '/status$');
          if (re.test(operation.path)) {this.setStatus(operation.value); break;}
          break;
        default:
          break;
      }
    }
  }

  requestCorpusExport() {
    nopaque.appClient.socket.emit('export_corpus', this.user.data.corpora[this.corpusId].id);
    nopaque.appClient.flash('Preparing your corpus export...', 'corpus');
    for (let exportCorpusTriggerElement of this.displayElement.querySelectorAll('.export-corpus-trigger')) {exportCorpusTriggerElement.classList.toggle('disabled', true);}
  }

  downloadCorpus() {
    nopaque.appClient.flash('Corpus export is done. Your corpus download is ready!', 'corpus');
    for (let exportCorpusTriggerElement of this.displayElement.querySelectorAll('.export-corpus-trigger')) {exportCorpusTriggerElement.classList.toggle('disabled', false);}
    // Little trick to call the download view after ziping has finished
    let fakeBtn = document.createElement('a');
    fakeBtn.href = `/corpora/${this.user.data.corpora[this.corpusId].id}/download`;
    fakeBtn.click();
  }

  setTitle(title) {
    for (let element of this.displayElement.querySelectorAll('.corpus-title')) {this.setElement(element, title);}
  }

  setTokenRatio(currentNrOfTokens, maxNrOfTokens) {
    let tokenRatio = `${currentNrOfTokens}/${maxNrOfTokens}`;
    for (let element of this.displayElement.querySelectorAll('.corpus-token-ratio')) {this.setElement(element, tokenRatio);}
  }

  setDescription(description) {
    for (let element of this.displayElement.querySelectorAll('.corpus-description')) {this.setElement(element, description);}
  }

  setStatus(status) {
    for (let element of this.displayElement.querySelectorAll('.analyse-corpus-trigger')) {
      if (['analysing', 'prepared', 'start analysis'].includes(status)) {
        element.classList.remove('disabled');
      } else {
        element.classList.add('disabled');
      }
    }
    for (let element of this.displayElement.querySelectorAll('.build-corpus-trigger')) {
      if (status === 'unprepared' && Object.values(this.user.data.corpora[this.corpusId].files).length > 0) {
        element.classList.remove('disabled');
      } else {
        element.classList.add('disabled');
      }
    }
    for (let element of this.displayElement.querySelectorAll('.corpus-status')) {this.setElement(element, status);}
    for (let exportCorpusTriggerElement of this.displayElement.querySelectorAll('.export-corpus-trigger')) {
      exportCorpusTriggerElement.classList.toggle('disabled', !['prepared', 'start analysis', 'stop analysis'].includes(status));
    }
    for (let element of this.displayElement.querySelectorAll('.status')) {element.dataset.status = status;}
    for (let element of this.displayElement.querySelectorAll('.status-spinner')) {
      if (['submitted', 'queued', 'running', 'canceling', 'start analysis', 'stop analysis'].includes(status)) {
        element.classList.remove('hide');
      } else {
        element.classList.add('hide');
      }
    }
  }

  setCreationDate(creationDateTimestamp) {
    let creationDate = new Date(creationDateTimestamp * 1000).toLocaleString("en-US");
    for (let element of this.displayElement.querySelectorAll('.corpus-creation-date')) {this.setElement(element, creationDate);}
  }

  setLastEditedDate(endDateTimestamp) {
    let endDate = new Date(endDateTimestamp * 1000).toLocaleString("en-US");
    for (let element of this.displayElement.querySelectorAll('.corpus-end-date')) {this.setElement(element, endDate);}
  }
}
