class CorpusList extends RessourceList {
  constructor(listElement, options = {}) {
    super(listElement, {...CorpusList.options, ...options});
    this.user.eventListeners.corpus.addEventListener((eventType, payload) => this.eventHandler(eventType, payload));
  }

  init() {
    super.init(this.user.data.corpora);
  }

  onclick(event) {
    let ressourceElement = event.target.closest('tr');
    if (ressourceElement === null) {return;}
    let corpusId = ressourceElement.dataset.id;
    let actionButtonElement = event.target.closest('.action-button');
    let action = (actionButtonElement === null) ? 'view' : actionButtonElement.dataset.action;
    switch (action) {
      case 'delete':
        let deleteModalHTML = `<div class="modal">
                                 <div class="modal-content">
                                   <h4>Confirm corpus deletion</h4>
                                   <p>Do you really want to delete the corpus <b>${this.user.data.corpora[corpusId].title}</b>? All files will be permanently deleted!</p>
                                 </div>
                                 <div class="modal-footer">
                                   <a href="#!" class="btn modal-close waves-effect waves-light">Cancel</a>
                                   <a class="btn modal-close red waves-effect waves-light" href="${this.user.data.corpora[corpusId].url}/delete"><i class="material-icons left">delete</i>Delete</a>
                                 </div>
                               </div>`;
        let deleteModalParentElement = document.querySelector('main');
        deleteModalParentElement.insertAdjacentHTML('beforeend', deleteModalHTML);
        let deleteModalElement = deleteModalParentElement.lastChild;
        let deleteModal = M.Modal.init(deleteModalElement, {onCloseEnd: () => {deleteModal.destroy(); deleteModalElement.remove();}});
        deleteModal.open();
        break;
      case 'view':
        if (corpusId !== '-1') {window.location.href = this.user.data.corpora[corpusId].url;}
        break;
      default:
        console.error(`Unknown action: ${action}`);
        break;
    }
  }

  patch(patch) {
    let id, match, re, valueName;
    for (let operation of patch) {
      switch(operation.op) {
        case 'add':
          // Matches the only paths that should be handled here: /corpora/{corpusId}
          re = /^\/corpora\/(\d+)$/;
          if (re.test(operation.path)) {this.add(operation.value);}
          break;
        case 'remove':
          // See case 'add' ;)
          re = /^\/corpora\/(\d+)$/;
          if (re.test(operation.path)) {
            [match, id] = operation.path.match(re);
            this.remove(id);
          }
          break;
        case 'replace':
          // Matches the only paths that should be handled here: /corpora/{corpusId}/{status || description || title}
          re = /^\/corpora\/(\d+)\/(status|description|title)$/;
          if (re.test(operation.path)) {
            [match, id, valueName] = operation.path.match(re);
            this.replace(id, valueName, operation.value);
          }
          break;
        default:
          break;
      }
    }
  }

  preprocessRessource(corpus) {
    return {id: corpus.id,
            status: corpus.status,
            description: corpus.description,
            title: corpus.title};
  }
}
CorpusList.options = {
  item: `<tr>
           <td><a class="btn-floating disabled"><i class="material-icons">book</i></a></td>
           <td><b class="title"></b><br><i class="description"></i></td>
           <td><span class="badge new status" data-badge-caption=""></span></td>
           <td class="right-align">
             <a class="action-button btn-floating red tooltipped waves-effect waves-light" data-action="delete" data-position="top" data-tooltip="Delete"><i class="material-icons">delete</i></a>
             <a class="action-button btn-floating tooltipped waves-effect waves-light" data-action="view" data-position="top" data-tooltip="View"><i class="material-icons">send</i></a>
           </td>
         </tr>`,
  valueNames: [{data: ['id']}, {name: 'status', attr: 'data-status'}, 'description', 'title']
};
