class JobList extends RessourceList {
  constructor(listElement, options = {}) {
    super(listElement, {...JobList.options, ...options});
    this.user.eventListeners.job.addEventListener((eventType, payload) => this.eventHandler(eventType, payload));
  }

  init() {
    super.init(this.user.data.jobs);
  }

  onclick(event) {
    let ressourceElement = event.target.closest('tr');
    if (ressourceElement === null) {return;}
    let jobId = ressourceElement.dataset.id;
    let actionButtonElement = event.target.closest('.action-button');
    let action = actionButtonElement === null ? 'view' : actionButtonElement.dataset.action;
    switch (action) {
      case 'delete':
        let deleteModalHTML = `<div class="modal">
                                 <div class="modal-content">
                                   <h4>Confirm job deletion</h4>
                                   <p>Do you really want to delete the job <b>${this.user.data.jobs[jobId].title}</b>? All files will be permanently deleted!</p>
                                 </div>
                                 <div class="modal-footer">
                                   <a href="#!" class="btn modal-close waves-effect waves-light">Cancel</a>
                                   <a class="btn modal-close red waves-effect waves-light" href="${this.user.data.jobs[jobId].url}/delete"><i class="material-icons left">delete</i>Delete</a>
                                 </div>
                               </div>`;
        let deleteModalParentElement = document.querySelector('main');
        deleteModalParentElement.insertAdjacentHTML('beforeend', deleteModalHTML);
        let deleteModalElement = deleteModalParentElement.lastChild;
        let deleteModal = M.Modal.init(deleteModalElement, {onCloseEnd: () => {deleteModal.destroy(); deleteModalElement.remove();}});
        deleteModal.open();
        break;
      case 'view':
        if (jobId !== '-1') {window.location.href = this.user.data.jobs[jobId].url;}
        break;
      default:
        console.error(`Unknown action: "${action}"`);
        break;
    }
  }

  patch(patch) {
    let id, match, re, valueName;
    for (let operation of patch) {
      switch(operation.op) {
        case 'add':
          // Matches the only paths that should be handled here: /jobs/{jobId}
          re = /^\/jobs\/(\d+)$/;
          if (re.test(operation.path)) {this.add(operation.value);}
          break;
        case 'remove':
          // See case add ;)
          re = /^\/jobs\/(\d+)$/;
          if (re.test(operation.path)) {
            [match, id] = operation.path.match(re);
            this.remove(id);
          }
          break;
        case 'replace':
          // Matches the only paths that should be handled here: /jobs/{jobId}/{service || status || description || title}
          re = /^\/jobs\/(\d+)\/(service|status|description|title)$/;
          if (re.test(operation.path)) {
            [match, id, valueName] = operation.path.match(re);
            this.replace(id, valueName, operation.value);
          }
          break;
        default:
          break;
      }
    }
  }

  preprocessRessource(job) {
    return {id: job.id,
            service: job.service,
            status: job.status,
            description: job.description,
            title: job.title};
  }
}
JobList.options = {
  item: `<tr>
           <td><a class="btn-floating disabled"><i class="material-icons service"></i></a></td>
           <td><b class="title"></b><br><i class="description"></i></td>
           <td><span class="badge new status" data-badge-caption=""></span></td>
           <td class="right-align">
             <a class="action-button btn-floating red tooltipped waves-effect waves-light" data-action="delete" data-position="top" data-tooltip="Delete"><i class="material-icons">delete</i></a>
             <a class="action-button btn-floating tooltipped waves-effect waves-light" data-action="view" data-position="top" data-tooltip="View"><i class="material-icons">send</i></a>
           </td>
         </tr>`,
  valueNames: [{data: ['id']}, {name: 'service', attr: 'data-service'}, {name: 'status', attr: 'data-status'}, 'description', 'title']
};
