class QueryResultList extends RessourceList {
  constructor(listElement, options = {}) {
    super(listElement, {...QueryResultList.options, ...options});
    this.user.eventListeners.queryResult.addEventListener((eventType, payload) => this.eventHandler(eventType, payload));
  }

  init() {
    super.init(this.user.data.query_results);
  }

  onclick(event) {
    let ressourceElement = event.target.closest('tr');
    if (ressourceElement === null) {return;}
    let queryResultId = ressourceElement.dataset.id;
    let actionButtonElement = event.target.closest('.action-button');
    let action = actionButtonElement === null ? 'view' : actionButtonElement.dataset.action;
    switch (action) {
      case 'delete':
        let deleteModalHTML = `<div class="modal">
                                 <div class="modal-content">
                                   <h4>Confirm query result deletion</h4>
                                   <p>Do you really want to delete the query result <b>${this.user.data.query_results[queryResultId].title}</b>? It will be permanently deleted!</p>
                                 </div>
                                 <div class="modal-footer">
                                   <a href="#!" class="btn modal-close waves-effect waves-light">Cancel</a>
                                   <a class="btn modal-close red waves-effect waves-light" href="${this.user.data.query_results[queryResultId].url}/delete"><i class="material-icons left">delete</i>Delete</a>
                                 </div>
                               </div>`;
        let deleteModalParentElement = document.querySelector('main');
        deleteModalParentElement.insertAdjacentHTML('beforeend', deleteModalHTML);
        let deleteModalElement = deleteModalParentElement.lastChild;
        let deleteModal = M.Modal.init(deleteModalElement, {onCloseEnd: () => {deleteModal.destroy(); deleteModalElement.remove();}});
        deleteModal.open();
        break;
      case 'view':
        if (queryResultId !== '-1') {window.location.href = this.user.data.query_results[queryResultId].url;}
        break;
      default:
        console.error(`Unknown action: "${action}"`);
        break;
    }
  }

  patch(patch) {
    let id, match, re, valueName;
    for (let operation of patch) {
      switch(operation.op) {
        case 'add':
          // Matches the only paths that should be handled here: /jobs/{jobId}
          re = /^\/query_results\/(\d+)$/;
          if (re.test(operation.path)) {this.add(operation.value);}
          break;
        case 'remove':
          // See case add ;)
          re = /^\/query_results\/(\d+)$/;
          if (re.test(operation.path)) {
            [match, id] = operation.path.match(re);
            this.remove(id);
          }
          break;
        case 'replace':
          // Matches the only paths that should be handled here: /jobs/{jobId}/{service || status || description || title}
          re = /^\/query_results\/(\d+)\/(corpus_title|description|query|title)$/;
          if (re.test(operation.path)) {
            [match, id, valueName] = operation.path.match(re);
            this.replace(id, valueName, operation.value);
          }
          break;
        default:
          break;
      }
    }
  }

  preprocessRessource(queryResult) {
    return {id: queryResult.id,
            corpus_title: queryResult.corpus_title,
            description: queryResult.description,
            query: queryResult.query,
            title: queryResult.title};
  }
}
QueryResultList.options = {
  item: `<tr>
           <td><b class="title"></b><br><i class="description"></i><br></td>
           <td><span class="corpus_title"></span><br><span class="query"></span></td>
           <td class="right-align">
             <a class="action-button btn-floating red tooltipped waves-effect waves-light" data-action="delete" data-position="top" data-tooltip="Delete"><i class="material-icons">delete</i></a>
             <a class="action-button btn-floating tooltipped waves-effect waves-light" data-action="view" data-position="top" data-tooltip="View"><i class="material-icons">send</i></a>
           </td>
         </tr>`,
  valueNames: [{data: ['id']}, 'corpus_title', 'description', 'query', 'title']
};
